@@ -1,18 +1,15 @@
-/* $Id: circular.c,v 1.3 2008/10/23 01:49:26 ellson Exp $ $Revision: 1.3 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 #include    "circular.h"
@@ -59,7 +56,6 @@ static void initGraphAttrs(Agraph_t * g, circ_state * state)
     }
     initBlocklist(&state->bl);
     state->orderCount = 1;
-    state->bcstack = mkStack();
     state->min_dist = min_dist;
     state->N_artpos = N_artpos;
     state->N_root = N_root;
@@ -74,7 +70,34 @@ static void initGraphAttrs(Agraph_t * g, circ_state * state)
 static void cleanup(block_t * root, circ_state * sp)
 {
     freeBlocktree(root);
-    freeStack(sp->bcstack);
+}
+
+static block_t*
+createOneBlock(Agraph_t * g, circ_state * state)
+{
+    Agraph_t *subg;
+    char name[SMALLBUF];
+    block_t *bp;
+    Agnode_t* n;
+
+    sprintf(name, "_block_%d", state->blockCount++);
+#ifdef WITH_CGRAPH
+    subg = agsubg(g, name, 1);
+#else
+    subg = agsubg(g, name);
+#endif
+    bp = mkBlock(subg);
+
+    for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+#ifdef WITH_CGRAPH
+	agsubnode(bp->sub_graph, n, 1);
+#else
+	aginsert(bp->sub_graph, n);
+#endif
+	BLOCK(n) = bp;
+    }
+
+    return bp;
 }
 
 /* circularLayout:
@@ -85,7 +108,7 @@ static void cleanup(block_t * root, circ_state * sp)
  * We make state static so that it keeps a record of block numbers used
  * in a graph; it gets reset when a new root graph is used.
  */
-void circularLayout(Agraph_t * g)
+void circularLayout(Agraph_t * g, Agraph_t* realg)
 {
     block_t *root;
     static circ_state state;
@@ -99,7 +122,10 @@ void circularLayout(Agraph_t * g)
 
     initGraphAttrs(g, &state);
 
-    root = createBlocktree(g, &state);
+    if (mapbool(agget(realg, "oneblock")))
+	root = createOneBlock(g, &state);
+    else
+	root = createBlocktree(g, &state);
     circPos(g, root, &state);
 
     cleanup(root, &state);
