@@ -1,18 +1,15 @@
-/* $Id: ns.c,v 1.18 2008/10/15 12:57:08 ellson Exp $ $Revision: 1.18 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 /* 
@@ -20,6 +17,7 @@
  */
 
 #include "render.h"
+#include <setjmp.h>
 
 static int init_graph(graph_t *);
 static void dfs_cutval(node_t * v, edge_t * par);
@@ -34,6 +32,7 @@ static void check_cycles(graph_t * g);
 #define SEQ(a,b,c)		(((a) <= (b)) && ((b) <= (c)))
 #define TREE_EDGE(e)	(ED_tree_index(e) >= 0)
 
+static jmp_buf jbuf;
 static graph_t *G;
 static int N_nodes, N_edges;
 static int Minrank, Maxrank;
@@ -46,8 +45,10 @@ static elist Tree_edge;
 static void add_tree_edge(edge_t * e)
 {
     node_t *n;
-    if (TREE_EDGE(e))
-	abort();
+    if (TREE_EDGE(e)) {
+	agerr(AGERR, "add_tree_edge: missing tree edge\n");
+	longjmp (jbuf, 1);
+    }
     ED_tree_index(e) = Tree_edge.size;
     Tree_edge.list[Tree_edge.size++] = e;
     if (ND_mark(agtail(e)) == FALSE)
@@ -58,14 +59,18 @@ static void add_tree_edge(edge_t * e)
     ND_mark(n) = TRUE;
     ND_tree_out(n).list[ND_tree_out(n).size++] = e;
     ND_tree_out(n).list[ND_tree_out(n).size] = NULL;
-    if (ND_out(n).list[ND_tree_out(n).size - 1] == 0)
-	abort();
+    if (ND_out(n).list[ND_tree_out(n).size - 1] == 0) {
+	agerr(AGERR, "add_tree_edge: empty outedge list\n");
+	longjmp (jbuf, 1);
+    }
     n = aghead(e);
     ND_mark(n) = TRUE;
     ND_tree_in(n).list[ND_tree_in(n).size++] = e;
     ND_tree_in(n).list[ND_tree_in(n).size] = NULL;
-    if (ND_in(n).list[ND_tree_in(n).size - 1] == 0)
-	abort();
+    if (ND_in(n).list[ND_tree_in(n).size - 1] == 0) {
+	agerr(AGERR, "add_tree_edge: empty inedge list\n");
+	longjmp (jbuf, 1);
+    }
 }
 
 static void exchange_tree_edges(edge_t * e, edge_t * f)
@@ -416,8 +421,10 @@ update(edge_t * e, edge_t * f)
 
     cutvalue = ED_cutvalue(e);
     lca = treeupdate(agtail(f), aghead(f), cutvalue, 1);
-    if (treeupdate(aghead(f), agtail(f), cutvalue, 0) != lca)
-	abort();
+    if (treeupdate(aghead(f), agtail(f), cutvalue, 0) != lca) {
+	agerr(AGERR, "update: mismatched lca in treeupdates\n");
+	longjmp (jbuf, 1);
+    }
     ED_cutvalue(f) = -cutvalue;
     ED_cutvalue(e) = 0;
     exchange_tree_edges(e, f);
@@ -596,13 +603,13 @@ graphSize (graph_t * g, int* nn, int* ne)
  *   Out and in edges lists stored in ND_out and ND_in, even if the node
  *  doesn't have any out or in edges.
  * The node rank values are stored in ND_rank.
- * Returns 0 if successful; returns 1 if not, the latter indicating that
- * the graph was not connected.
+ * Returns 0 if successful; returns 1 if `he graph was not connected;
+ * returns 2 if something seriously wrong;
  */
-int rank(graph_t * g, int balance, int maxiter)
+int rank2(graph_t * g, int balance, int maxiter, int search_size)
 {
     int iter = 0, feasible;
-    char *s, *ns = "network simplex: ";
+    char *ns = "network simplex: ";
     edge_t *e, *f;
 
 #ifdef DEBUG
@@ -623,11 +630,15 @@ int rank(graph_t * g, int balance, int maxiter)
 	return 0;
     }
 
-    if ((s = agget(g, "searchsize")))
-	Search_size = atoi(s);
+    if (search_size >= 0)
+	Search_size = search_size;
     else
 	Search_size = SEARCHSIZE;
 
+    if (setjmp (jbuf)) {
+	return 2;
+    }
+
     if (feasible_tree()) {
 	freeTreeList (g);
 	return 1;
@@ -655,6 +666,7 @@ int rank(graph_t * g, int balance, int maxiter)
 	break;
     default:
 	scan_and_normalize();
+	freeTreeList (G);
 	break;
     }
     if (Verbose) {
@@ -666,6 +678,19 @@ int rank(graph_t * g, int balance, int maxiter)
     return 0;
 }
 
+int rank(graph_t * g, int balance, int maxiter)
+{
+    char *s;
+    int search_size;
+
+    if ((s = agget(g, "searchsize")))
+	search_size = atoi(s);
+    else
+	search_size = SEARCHSIZE;
+
+    return rank2 (g, balance, maxiter, search_size);
+}
+
 /* set cut value of f, assuming values of edges on one side were already set */
 static void x_cutval(edge_t * f)
 {
@@ -770,12 +795,12 @@ void tchk(void)
 
     n_cnt = 0;
     e_cnt = 0;
-    for (n = GD_nlist(G); n; n = ND_next(n)) {
+    for (n = agfstnode(G); n; n = agnxtnode(G, n)) {
 	n_cnt++;
 	for (i = 0; (e = ND_tree_out(n).list[i]); i++) {
 	    e_cnt++;
 	    if (SLACK(e) > 0)
-		fprintf(stderr, "not a tight tree %lx", (unsigned long int)e);
+		fprintf(stderr, "not a tight tree %p", e);
 	}
     }
     if ((n_cnt != Tree_node.size) || (e_cnt != Tree_edge.size))
@@ -788,7 +813,7 @@ void check_cutvalues(void)
     edge_t *e;
     int i, save;
 
-    for (v = GD_nlist(G); v; v = ND_next(v)) {
+    for (v = agfstnode(G); v; v = agnxtnode(G, v)) {
 	for (i = 0; (e = ND_tree_out(v).list[i]); i++) {
 	    save = ED_cutvalue(e);
 	    x_cutval(e);
@@ -800,12 +825,12 @@ void check_cutvalues(void)
 
 int check_ranks(void)
 {
-    int i, cost = 0;
+    int cost = 0;
     node_t *n;
     edge_t *e;
 
-    for (n = GD_nlist(G); n; n = ND_next(n)) {
-	for (i = 0; (e = ND_out(n).list[i]); i++) {
+    for (n = agfstnode(G); n; n = agnxtnode(G, n)) {
+	for (e = agfstout(G, n); e; e = agnxtout(G, e)) {
 	    cost += (ED_weight(e)) * abs(LENGTH(e));
 	    if (ND_rank(aghead(e)) - ND_rank(agtail(e)) - ED_minlen(e) < 0)
 		abort();
@@ -821,7 +846,7 @@ void checktree(void)
     node_t *v;
     edge_t *e;
 
-    for (v = GD_nlist(G); v; v = ND_next(v)) {
+    for (v = agfstnode(G); v; v = agnxtnode(G, v)) {
 	for (i = 0; (e = ND_tree_out(v).list[i]); i++)
 	    n++;
 	if (i != ND_tree_out(v).size)
@@ -843,11 +868,42 @@ void check_fast_node(node_t * n)
     assert(nptr != NULL);
 }
 
-static node_t *checkdfs(node_t * n)
+static void dump_graph (graph_t* g)
 {
     int i;
+    edge_t *e;
+    node_t *n,*w;
+    FILE* fp = fopen ("ns.gv", "w");
+    fprintf (fp, "digraph %s {\n", g->name);
+    for (n = GD_nlist(g); n; n = ND_next(n)) {
+	if (streq(n->name,"virtual"))
+	    fprintf (fp, "  \"%p\"\n", n);
+	else
+	    fprintf (fp, "  \"%s\"\n", n->name);
+    }
+    for (n = GD_nlist(g); n; n = ND_next(n)) {
+	for (i = 0; (e = ND_out(n).list[i]); i++) {
+	    if (streq(n->name,"virtual"))
+		fprintf (fp, "  \"%p\"", n);
+	    else
+		fprintf (fp, "  \"%s\"", n->name);
+	    w = aghead(e);
+	    if (streq(w->name,"virtual"))
+		fprintf (fp, " -> \"%p\"\n", w);
+	    else
+		fprintf (fp, " -> \"%s\"\n", w->name);
+	}
+    }
+
+    fprintf (fp, "}\n");
+    fclose (fp);
+}
+
+static node_t *checkdfs(graph_t* g, node_t * n)
+{
     edge_t *e;
     node_t *w,*x;
+    int i;
 
     if (ND_mark(n))
 	return 0;
@@ -856,6 +912,7 @@ static node_t *checkdfs(node_t * n)
     for (i = 0; (e = ND_out(n).list[i]); i++) {
 	w = aghead(e);
 	if (ND_onstack(w)) {
+	    dump_graph (g);
 	    fprintf(stderr, "cycle: last edge %lx %s(%lx) %s(%lx)\n",
 		(unsigned long int)e,
 	       	agnameof(n), (unsigned long int)n,
@@ -864,7 +921,7 @@ static node_t *checkdfs(node_t * n)
 	}
 	else {
 	    if (ND_mark(w) == FALSE) {
-		x = checkdfs(w);
+		x = checkdfs(g, w);
 		if (x) {
 		    fprintf(stderr,"unwind %lx %s(%lx)\n",
 			(unsigned long int)e,
@@ -888,6 +945,6 @@ void check_cycles(graph_t * g)
     for (n = GD_nlist(g); n; n = ND_next(n))
 	ND_mark(n) = ND_onstack(n) = FALSE;
     for (n = GD_nlist(g); n; n = ND_next(n))
-	checkdfs(n);
+	checkdfs(g, n);
 }
 #endif				/* DEBUG */
