@@ -1,18 +1,15 @@
-/* $Id: routespl.c,v 1.27 2009/11/05 19:30:18 ellson Exp $ $Revision: 1.27 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -20,6 +17,7 @@
 
 #include "render.h"
 #include "pathplan.h"
+#include <setjmp.h>
 
 #ifdef UNUSED
 static box *bs = NULL;
@@ -45,8 +43,8 @@ static int polypointn;        /* size of polypoints[] */
 static Pedge_t *edges;        /* polygon edges passed to Proutespline */
 static int edgen;             /* size of edges[] */
 
-static void checkpath(int, boxf*, path*);
-static void mkspacep(int size);
+static int checkpath(int, boxf*, path*);
+static int mkspacep(int size);
 static void printpath(path * pp);
 #ifdef DEBUG
 static void printboxes(int boxn, boxf* boxes)
@@ -223,17 +221,76 @@ static int debugleveln(edge_t* realedge, int i)
 }
 #endif  /* DEBUG */
 
+
+
+/* simpleSplineRoute:
+ * Given a simple (ccw) polygon, route an edge from tp to hp.
+ */
+pointf*
+simpleSplineRoute (pointf tp, pointf hp, Ppoly_t poly, int* n_spl_pts,
+    int polyline)
+{
+    Ppolyline_t pl, spl;
+    Ppoint_t eps[2];
+    Pvector_t evs[2];
+    int i;
+
+    eps[0].x = tp.x;
+    eps[0].y = tp.y;
+    eps[1].x = hp.x;
+    eps[1].y = hp.y;
+    if (Pshortestpath(&poly, eps, &pl) < 0)
+        return NULL;
+
+    if (polyline)
+	make_polyline (pl, &spl);
+    else {
+	if (poly.pn > edgen) {
+	    edges = ALLOC(poly.pn, edges, Pedge_t);
+	    edgen = poly.pn;
+	}
+	for (i = 0; i < poly.pn; i++) {
+	    edges[i].a = poly.ps[i];
+	    edges[i].b = poly.ps[(i + 1) % poly.pn];
+	}
+#if 0
+	if (pp->start.constrained) {
+	    evs[0].x = cos(pp->start.theta);
+	    evs[0].y = sin(pp->start.theta);
+	} else
+#endif
+	    evs[0].x = evs[0].y = 0;
+#if 0
+	if (pp->end.constrained) {
+	    evs[1].x = -cos(pp->end.theta);
+	    evs[1].y = -sin(pp->end.theta);
+	} else
+#endif
+	    evs[1].x = evs[1].y = 0;
+	if (Proutespline(edges, poly.pn, pl, evs, &spl) < 0)
+            return NULL;
+    }
+
+    if (mkspacep(spl.pn))
+	return NULL;
+    for (i = 0; i < spl.pn; i++) {
+        ps[i] = spl.ps[i];
+    }
+    *n_spl_pts = spl.pn;
+    return ps;
+}
+
 /* routesplinesinit:
  * Data initialized once until matching call to routeplineterm
  * Allows recursive calls to dot
  */
-void
+int
 routesplinesinit()
 {
-    if (++routeinit > 1) return;
+    if (++routeinit > 1) return 0;
     if (!(ps = N_GNEW(PINC, pointf))) {
-	agerr(AGERR, "cannot allocate ps\n");
-	abort();
+	agerr(AGERR, "routesplinesinit: cannot allocate ps\n");
+	return 1;
     }
     maxpn = PINC;
 #ifdef DEBUG
@@ -250,6 +307,7 @@ routesplinesinit()
     nboxes = 0;
     if (Verbose)
 	start_timer();
+    return 0;
 }
 
 void routesplinesterm()
@@ -265,6 +323,70 @@ void routesplinesterm()
 		nedges, nboxes, elapsed_sec());
 }
 
+static void
+limitBoxes (boxf* boxes, int boxn, pointf *pps, int pn, int delta)
+{
+    int bi, si, splinepi;
+    double t;
+    pointf sp[4];
+    int num_div = delta * boxn;
+
+    for (splinepi = 0; splinepi + 3 < pn; splinepi += 3) {
+	for (si = 0; si <= num_div; si++) {
+	    t = si / (double)num_div;
+	    sp[0] = pps[splinepi];
+	    sp[1] = pps[splinepi + 1];
+	    sp[2] = pps[splinepi + 2];
+	    sp[3] = pps[splinepi + 3];
+	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
+	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
+	    sp[1].x = sp[1].x + t * (sp[2].x - sp[1].x);
+	    sp[1].y = sp[1].y + t * (sp[2].y - sp[1].y);
+	    sp[2].x = sp[2].x + t * (sp[3].x - sp[2].x);
+	    sp[2].y = sp[2].y + t * (sp[3].y - sp[2].y);
+ 	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
+	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
+	    sp[1].x = sp[1].x + t * (sp[2].x - sp[1].x);
+	    sp[1].y = sp[1].y + t * (sp[2].y - sp[1].y);
+	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
+	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
+	    for (bi = 0; bi < boxn; bi++) {
+/* this tested ok on 64bit machines, but on 32bit we need this FUDGE
+ *     or graphs/directed/records.gv fails */
+#define FUDGE .0001
+		if (sp[0].y <= boxes[bi].UR.y+FUDGE && sp[0].y >= boxes[bi].LL.y-FUDGE) {
+		    if (boxes[bi].LL.x > sp[0].x)
+			boxes[bi].LL.x = sp[0].x;
+		    if (boxes[bi].UR.x < sp[0].x)
+			boxes[bi].UR.x = sp[0].x;
+		}
+	    }
+	}
+    }
+}
+
+#define INIT_DELTA 10 
+#define LOOP_TRIES 15  /* number of times to try to limiting boxes to regain space, using smaller divisions */
+
+/* routesplines:
+ * Route a path using the path info in pp. This includes start and end points
+ * plus a collection of contiguous boxes contain the terminal points. The boxes
+ * are converted into a containing polygon. A shortest path is constructed within
+ * the polygon from between the terminal points. If polyline is true, this path
+ * is converted to a spline representation. Otherwise, we call the path planner to
+ * convert the polyline into a smooth spline staying within the polygon. In both
+ * cases, the function returns an array of the computed control points. The number
+ * of these points is given in npoints.
+ *
+ * Note that the returned points are stored in a single array, so the points must be
+ * used before another call to this function.
+ *
+ * During cleanup, the function determines the x-extent of the spline in the box, so
+ * the box can be shrunk to the minimum width. The extra space can then be used by other
+ * edges. 
+ *
+ * If a catastrophic error, return NULL.
+ */
 static pointf *_routesplines(path * pp, int *npoints, int polyline)
 {
     Ppoly_t poly;
@@ -273,14 +395,13 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
     Ppoint_t eps[2];
     Pvector_t evs[2];
     int edgei, prev, next;
-    pointf sp[4];
-    int pi, bi, si;
-    double t;
+    int pi, bi;
     boxf *boxes;
     int boxn;
     edge_t* realedge;
     int flip;
-    int delta = 10;
+    int loopcnt, delta = INIT_DELTA;
+    boolean unbounded;
 
     nedges++;
     nboxes += pp->nbox;
@@ -293,13 +414,14 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	 realedge = ED_to_orig(realedge));
     if (!realedge) {
 	agerr(AGERR, "in routesplines, cannot find NORMAL edge\n");
-	abort();
+	return NULL;
     }
 
     boxes = pp->boxes;
     boxn = pp->nbox;
 
-    checkpath(boxn, boxes, pp);
+    if (checkpath(boxn, boxes, pp))
+	return NULL;
 
 #ifdef DEBUG
     if (debugleveln(realedge, 1))
@@ -354,8 +476,10 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 		polypoints[pi++].y = boxes[bi].LL.y;
 	    } 
 	    else {
-		if (!(prev == -1 && next == -1))
-		    abort();
+		if (!(prev == -1 && next == -1)) {
+		    agerr(AGERR, "in routesplines, illegal values of prev %d and next %d, line %d\n", prev, next, __LINE__);
+		    return NULL;
+		}
 	    }
 	}
 	for (bi = boxn - 1; bi >= 0; bi--) {
@@ -386,9 +510,8 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	    else {
 		if (!(prev == -1 && next == -1)) {
 		    /* it went badly, e.g. degenerate box in boxlist */
-		    *npoints = 0;
-		    abort();	/* for correctness sake, it's best to just stop */
-		    return ps;	/* could also be reported as a lost edge (no spline) */
+		    agerr(AGERR, "in routesplines, illegal values of prev %d and next %d, line %d\n", prev, next, __LINE__);
+		    return NULL; /* for correctness sake, it's best to just stop */
 		}
 		polypoints[pi].x = boxes[bi].UR.x;
 		polypoints[pi++].y = boxes[bi].LL.y;
@@ -402,7 +525,8 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	}
     }
     else {
-	abort();
+	agerr(AGERR, "in routesplines, edge is a loop at %s\n", agnameof(aghead(realedge)));
+	return NULL;
     }
 
     if (flip) {
@@ -421,8 +545,10 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
     poly.ps = polypoints, poly.pn = pi;
     eps[0].x = pp->start.p.x, eps[0].y = pp->start.p.y;
     eps[1].x = pp->end.p.x, eps[1].y = pp->end.p.y;
-    if (Pshortestpath(&poly, eps, &pl) == -1)
-	abort();
+    if (Pshortestpath(&poly, eps, &pl) < 0) {
+	agerr(AGERR, "in routesplines, Pshortestpath failed\n");
+	return NULL;
+    }
 #ifdef DEBUG
     if (debugleveln(realedge, 3)) {
 	psprintpoly(poly);
@@ -453,8 +579,10 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	} else
 	    evs[1].x = evs[1].y = 0;
 
-	if (Proutespline(edges, poly.pn, pl, evs, &spl) == -1)
-	    abort();
+	if (Proutespline(edges, poly.pn, pl, evs, &spl) < 0) {
+	    agerr(AGERR, "in routesplines, Proutespline failed\n");
+	    return NULL;
+	}
 #ifdef DEBUG
 	if (debugleveln(realedge, 3)) {
 	    psprintspline(spl);
@@ -462,61 +590,52 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	}
 #endif
     }
-    mkspacep(spl.pn);
+    if (mkspacep(spl.pn))
+	return NULL;  /* Bailout if no memory left */
+
     for (bi = 0; bi < boxn; bi++) {
 	boxes[bi].LL.x = INT_MAX;
 	boxes[bi].UR.x = INT_MIN;
     }
+    unbounded = TRUE;
     for (splinepi = 0; splinepi < spl.pn; splinepi++) {
 	ps[splinepi] = spl.ps[splinepi];
     }
-REDO:
-    for (splinepi = 0; splinepi + 3 < spl.pn; splinepi += 3) {
-	int num_div = delta * boxn;
-	for (si = 0; si <= num_div; si++) {
-	    t = si / (double)num_div;
-	    sp[0] = ps[splinepi];
-	    sp[1] = ps[splinepi + 1];
-	    sp[2] = ps[splinepi + 2];
-	    sp[3] = ps[splinepi + 3];
-	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
-	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
-	    sp[1].x = sp[1].x + t * (sp[2].x - sp[1].x);
-	    sp[1].y = sp[1].y + t * (sp[2].y - sp[1].y);
-	    sp[2].x = sp[2].x + t * (sp[3].x - sp[2].x);
-	    sp[2].y = sp[2].y + t * (sp[3].y - sp[2].y);
-	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
-	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
-	    sp[1].x = sp[1].x + t * (sp[2].x - sp[1].x);
-	    sp[1].y = sp[1].y + t * (sp[2].y - sp[1].y);
-	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
-	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
-	    for (bi = 0; bi < boxn; bi++) {
-/* this tested ok on 64bit machines, but on 32bit we need this FUDGE
- *     or graphs/directed/records.gv fails */
-#define FUDGE .0001
-		if (sp[0].y <= boxes[bi].UR.y+FUDGE && sp[0].y >= boxes[bi].LL.y-FUDGE) {
-		    if (boxes[bi].LL.x > sp[0].x)
-			boxes[bi].LL.x = sp[0].x;
-		    if (boxes[bi].UR.x < sp[0].x)
-			boxes[bi].UR.x = sp[0].x;
-		}
-	    }
-	}
-    }
+
+    for (loopcnt = 0; unbounded && (loopcnt < LOOP_TRIES); loopcnt++) {
+	limitBoxes (boxes, boxn, ps, spl.pn, delta);
+
     /* The following check is necessary because if a box is not very 
      * high, it is possible that the sampling above might miss it.
      * Therefore, we make the sample finer until all boxes have
      * valid values. cf. bug 456. Would making sp[] pointfs help?
      */
-    for (bi = 0; bi < boxn; bi++) {
+	for (bi = 0; bi < boxn; bi++) {
 	/* these fp equality tests are used only to detect if the
 	 * values have been changed since initialization - ok */
-	if ((boxes[bi].LL.x == INT_MAX) || (boxes[bi].UR.x == INT_MIN)) {
-	    delta *= 2;
-	    goto REDO;
+	    if ((boxes[bi].LL.x == INT_MAX) || (boxes[bi].UR.x == INT_MIN)) {
+		delta *= 2; /* try again with a finer interval */
+		if (delta > INT_MAX/boxn) /* in limitBoxes, boxn*delta must fit in an int, so give up */
+		    loopcnt = LOOP_TRIES;
+		break;
+	    }
 	}
+	if (bi == boxn)
+	    unbounded = FALSE;
     }
+    if (unbounded) {  
+	/* Either an extremely short, even degenerate, box, or some failure with the path
+         * planner causing the spline to miss some boxes. In any case, use the shortest path 
+	 * to bound the boxes. This will probably mean a bad edge, but we avoid an infinite
+	 * loop and we can see the bad edge, and even use the showboxes scaffolding.
+	 */
+	Ppolyline_t polyspl;
+	agerr(AGWARN, "Unable to reclaim box space in spline routing for edge \"%s\" -> \"%s\". Something is probably seriously wrong.\n", agnameof(agtail(realedge)), agnameof(aghead(realedge)));
+	make_polyline (pl, &polyspl);
+	limitBoxes (boxes, boxn, polyspl.ps, polyspl.pn, INIT_DELTA);
+	free (polyspl.ps);
+    }
+
     *npoints = spl.pn;
 
 #ifdef DEBUG
@@ -562,8 +681,10 @@ static int overlap(int i0, int i1, int j0, int j1)
  * audit process in the 5E control program - if you've given up on
  * fixing all the bugs, at least try to engineer around them!
  * in postmodern CS, we could call this "self-healing code."
+ *
+ * Return 1 on failure; 0 on success.
  */
-static void checkpath(int boxn, boxf* boxes, path* thepath)
+static int checkpath(int boxn, boxf* boxes, path* thepath)
 {
     boxf *ba, *bb;
     int bi, i, errs, l, r, d, u;
@@ -588,7 +709,7 @@ static void checkpath(int boxn, boxf* boxes, path* thepath)
     if (ba->LL.x > ba->UR.x || ba->LL.y > ba->UR.y) {
 	agerr(AGERR, "in checkpath, box 0 has LL coord > UR coord\n");
 	printpath(thepath);
-	abort();
+	return 1;
     }
     for (bi = 0; bi < boxn - 1; bi++) {
 	ba = &boxes[bi], bb = &boxes[bi + 1];
@@ -596,7 +717,7 @@ static void checkpath(int boxn, boxf* boxes, path* thepath)
 	    agerr(AGERR, "in checkpath, box %d has LL coord > UR coord\n",
 		  bi + 1);
 	    printpath(thepath);
-	    abort();
+	    return 1;
 	}
 	l = (ba->UR.x < bb->LL.x) ? 1 : 0;
 	r = (ba->LL.x > bb->UR.x) ? 1 : 0;
@@ -718,15 +839,21 @@ static void checkpath(int boxn, boxf* boxes, path* thepath)
 	abort();
 #endif
     }
+    return 0;
 }
 
-static void mkspacep(int size)
+static int mkspacep(int size)
 {
     if (size > maxpn) {
 	int newmax = maxpn + (size / PINC + 1) * PINC;
 	ps = RALLOC(newmax, ps, pointf);
+	if (!ps) {
+	    agerr(AGERR, "cannot re-allocate ps\n");
+	    return 1;
+	}
 	maxpn = newmax;
     }
+    return 0;
 }
 
 static void printpath(path * pp)
@@ -752,3 +879,138 @@ static void printpath(path * pp)
 	    pp->end.constrained ? "constrained" : "not constrained");
 }
 
+static pointf get_centroid(Agraph_t *g)
+{
+    int     cnt = 0;
+    static pointf   sum = {0.0, 0.0};
+    static Agraph_t *save;
+    Agnode_t *n;
+
+    sum.x = (GD_bb(g).LL.x + GD_bb(g).UR.x) / 2.0;
+    sum.y = (GD_bb(g).LL.y + GD_bb(g).UR.y) / 2.0;
+    return sum;
+
+    if (save == g) return sum;
+    save = g;
+    for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+        sum.x += ND_pos(n)[0];
+        sum.y += ND_pos(n)[1];
+        cnt++;
+    }
+    sum.x = sum.x / cnt;
+    sum.y = sum.y / cnt;
+    return sum;
+}
+
+static void bend(pointf spl[4], pointf centroid)
+{
+    pointf  midpt,a;
+    double  r;
+    double  dist,dx,dy;
+
+    midpt.x = (spl[0].x + spl[3].x)/2.0;
+    midpt.y = (spl[0].y + spl[3].y)/2.0;
+    dx = (spl[3].x - spl[0].x);
+    dy = (spl[3].y - spl[0].y);
+    dist = sqrt(dx*dx + dy*dy);
+    r = dist/5.0;
+    {
+        double vX = centroid.x - midpt.x;
+        double vY = centroid.y - midpt.y;
+        double magV = sqrt(vX*vX + vY*vY);
+        a.x = midpt.x - vX / magV * r;      /* + would be closest point */
+        a.y = midpt.y - vY / magV * r;
+    }
+    /* this can be improved */
+    spl[1].x = spl[2].x = a.x;
+    spl[1].y = spl[2].y = a.y;
+}
+
+/* makeStraightEdge:
+ *
+ * FIX: handle ports on boundary?
+ */
+void 
+makeStraightEdge(graph_t * g, edge_t * e, int et, splineInfo* sinfo)
+{
+    pointf dumb[4];
+    node_t *n = agtail(e);
+    node_t *head = aghead(e);
+    int e_cnt = ED_count(e);
+    int curved = (et == ET_CURVED);
+    pointf perp;
+    pointf del;
+    edge_t *e0;
+    int i, j, xstep, dx;
+    double l_perp;
+    pointf dumber[4];
+    pointf p, q;
+
+    p = dumb[1] = dumb[0] = add_pointf(ND_coord(n), ED_tail_port(e).p);
+    q = dumb[2] = dumb[3] = add_pointf(ND_coord(head), ED_head_port(e).p);
+    if ((e_cnt == 1) || Concentrate) {
+	if (curved) bend(dumb,get_centroid(g));
+	clip_and_install(e, aghead(e), dumb, 4, sinfo);
+	addEdgeLabels(g, e, p, q);
+	return;
+    }
+
+    e0 = e;
+    if (APPROXEQPT(dumb[0], dumb[3], MILLIPOINT)) {
+	/* degenerate case */
+	dumb[1] = dumb[0];
+	dumb[2] = dumb[3];
+	del.x = 0;
+	del.y = 0;
+    }
+    else {
+        perp.x = dumb[0].y - dumb[3].y;
+        perp.y = dumb[3].x - dumb[0].x;
+	l_perp = LEN(perp.x, perp.y);
+	xstep = GD_nodesep(g->root);
+	dx = xstep * (e_cnt - 1) / 2;
+	dumb[1].x = dumb[0].x + (dx * perp.x) / l_perp;
+	dumb[1].y = dumb[0].y + (dx * perp.y) / l_perp;
+	dumb[2].x = dumb[3].x + (dx * perp.x) / l_perp;
+	dumb[2].y = dumb[3].y + (dx * perp.y) / l_perp;
+	del.x = -xstep * perp.x / l_perp;
+	del.y = -xstep * perp.y / l_perp;
+    }
+
+    for (i = 0; i < e_cnt; i++) {
+	if (aghead(e0) == head) {
+	    p = dumb[0];
+	    q = dumb[3];
+	    for (j = 0; j < 4; j++) {
+		dumber[j] = dumb[j];
+	    }
+	} else {
+	    p = dumb[3];
+	    q = dumb[0];
+	    for (j = 0; j < 4; j++) {
+		dumber[3 - j] = dumb[j];
+	    }
+	}
+	if (et == ET_PLINE) {
+	    Ppoint_t pts[4];
+	    Ppolyline_t spl, line;
+
+	    line.pn = 4;
+	    line.ps = pts;
+	    for (j=0; j < 4; j++) {
+		pts[j] = dumber[j];
+	    }
+	    make_polyline (line, &spl);
+	    clip_and_install(e0, aghead(e0), spl.ps, spl.pn, sinfo);
+	}
+	else
+	    clip_and_install(e0, aghead(e0), dumber, 4, sinfo);
+
+	addEdgeLabels(g, e0, p, q);
+	e0 = ED_to_virt(e0);
+	dumb[1].x += del.x;
+	dumb[1].y += del.y;
+	dumb[2].x += del.x;
+	dumb[2].y += del.y;
+    }
+}
