@@ -1,24 +1,36 @@
-/* $Id: dotinit.c,v 1.20 2009/06/03 01:10:52 ellson Exp $ $Revision: 1.20 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 #include <time.h>
 #include "dot.h"
 #include "aspect.h"
 
+#ifdef WITH_CGRAPH
+static void
+dot_init_subg(graph_t * g)
+{
+    graph_t* subg;
+
+    if ((g != agroot(g)))
+	agbindrec(g, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+    for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
+	dot_init_subg(subg);
+    }
+}
+#endif
+
+
 static void 
 dot_init_node(node_t * n)
 {
@@ -44,7 +56,7 @@ dot_init_edge(edge_t * e)
 #endif /* WITH_CGRAPH */
     common_init_edge(e);
 
-    ED_weight(e) = late_double(e, E_weight, 1.0, 0.0);
+    ED_weight(e) = late_int(e, E_weight, 1, 0);
     tailgroup = late_string(agtail(e), N_group, "");
     headgroup = late_string(aghead(e), N_group, "");
     ED_count(e) = ED_xpenalty(e) = 1;
@@ -102,7 +114,7 @@ dot_cleanup_node(node_t * n)
 #ifndef WITH_CGRAPH
     memset(&(n->u), 0, sizeof(Agnodeinfo_t));
 #else /* WITH_CGRAPH */
-	agclean(agraphof(n), AGNODE,"Agnodeinfo_t");	
+    agdelrec(n, "Agnodeinfo_t");	
 #endif /* WITH_CGRAPH */
 }
 
@@ -114,11 +126,17 @@ static void free_virtual_edge_list(node_t * n)
     for (i = ND_in(n).size - 1; i >= 0; i--) {
 	e = ND_in(n).list[i];
 	delete_fast_edge(e);
+#ifdef WITH_CGRAPH
+	free(e->base.data);
+#endif
 	free(e);
     }
     for (i = ND_out(n).size - 1; i >= 0; i--) {
 	e = ND_out(n).list[i];
 	delete_fast_edge(e);
+#ifdef WITH_CGRAPH
+	free(e->base.data);
+#endif
 	free(e);
     }
 }
@@ -133,6 +151,9 @@ static void free_virtual_node_list(node_t * vn)
 	if (ND_node_type(vn) == VIRTUAL) {
 	    free_list(ND_out(vn));
 	    free_list(ND_in(vn));
+#ifdef WITH_CGRAPH
+	    free(vn->base.data);
+#endif
 	    free(vn);
 	}
 	vn = next_vn;
@@ -142,26 +163,38 @@ static void free_virtual_node_list(node_t * vn)
 static void 
 dot_cleanup_graph(graph_t * g)
 {
-    int i, c;
+    int i;
+#ifndef WITH_CGRAPH
     graph_t *clust;
-
+    int c;
     for (c = 1; c <= GD_n_cluster(g); c++) {
 	clust = GD_clust(g)[c];
-	GD_cluster_was_collapsed(clust) = FALSE;
+	GD_parent(clust) = NULL;
 	dot_cleanup(clust);
     }
+#else
+    graph_t *subg;
+    for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
+	dot_cleanup_graph(subg);
+    }
+#endif
+    if (GD_clust(g)) free (GD_clust(g));
+    if (GD_rankleader(g)) free (GD_rankleader(g));
 
     free_list(GD_comp(g));
-    if ((g == agroot(g)) && GD_rank(g)) {
+    if (GD_rank(g)) {
 	for (i = GD_minrank(g); i <= GD_maxrank(g); i++)
-	    free(GD_rank(g)[i].v);
-	free(GD_rank(g));
+	    free(GD_rank(g)[i].av);
+	if (GD_minrank(g) == -1)
+	    free(GD_rank(g)-1);
+	else
+	    free(GD_rank(g));
     }
     if (g != agroot(g)) 
 #ifndef WITH_CGRAPH
 	memset(&(g->u), 0, sizeof(Agraphinfo_t));
 #else /* WITH_CGRAPH */
-	agclean(g,AGRAPH,"Agraphinfo_t");
+	agdelrec(g,"Agraphinfo_t");
 #endif /* WITH_CGRAPH */
 }
 
@@ -247,28 +280,120 @@ setAspect (Agraph_t * g, aspect_t* adata)
     return adata;
 }
 
+#ifdef WITH_CGRAPH
+static void
+remove_from_rank (Agraph_t * g, Agnode_t* n)
+{
+    Agnode_t* v = NULL;
+    int j, rk = ND_rank(n);
+
+    for (j = 0; j < GD_rank(g)[rk].n; j++) {
+	v = GD_rank(g)[rk].v[j];
+	if (v == n) {
+	    for (j++; j < GD_rank(g)[rk].n; j++) {
+		GD_rank(g)[rk].v[j-1] = GD_rank(g)[rk].v[j];
+	    }
+	    GD_rank(g)[rk].n--;
+	    break;
+	}
+    }
+    assert (v == n);  /* if found */
+}
+
+/* removeFill:
+ * This removes all of the fill nodes added in mincross.
+ * It appears to be sufficient to remove them only from the
+ * rank array and fast node list of the root graph.
+ */
+static void
+removeFill (Agraph_t * g)
+{
+    Agnode_t* n;
+    Agnode_t* nxt;
+    Agraph_t* sg = agsubg (g, "_new_rank", 0);
+
+    if (!sg) return;
+    for (n = agfstnode(sg); n; n = nxt) {
+	nxt = agnxtnode(sg, n);
+	delete_fast_node (g, n);
+	remove_from_rank (g, n);
+	dot_cleanup_node (n);
+	agdelnode(g, n);
+    }
+    agdelsubg (g, sg);
+
+}
+#endif
+
+#ifndef WITH_CGRAPH
+#define ag_xset(x,a,v) agxset(x,a->index,v)
+#else /* WITH_CGRAPH */
+#define ag_xset(x,a,v) agxset(x,a,v)
+#define agnodeattr(g,n,v) agattr(g,AGNODE,n,v)
+#endif /* WITH_CGRAPH */
+
+static void
+attach_phase_attrs (Agraph_t * g, int maxphase)
+{
+    Agsym_t* rk = agnodeattr(g,"rank","");
+    Agsym_t* order = agnodeattr(g,"order","");
+    Agnode_t* n;
+    char buf[BUFSIZ];
+
+    for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+	if (maxphase >= 1) {
+	    sprintf(buf, "%d", ND_rank(n));
+	    ag_xset(n,rk,buf);
+	}
+	if (maxphase >= 2) {
+	    sprintf(buf, "%d", ND_order(n));
+	    ag_xset(n,order,buf);
+	}
+    }
+}
+
 void dot_layout(Agraph_t * g)
 {
     aspect_t aspect;
     aspect_t* asp;
+    int maxphase = late_int(g, agfindgraphattr(g,"phase"), -1, 1);
 
     setEdgeType (g, ET_SPLINE);
     asp = setAspect (g, &aspect);
 
+#ifdef WITH_CGRAPH
+    dot_init_subg(g);
+#endif
+
     dot_init_node_edge(g);
 
     do {
         dot_rank(g, asp);
+	if (maxphase == 1) {
+	    attach_phase_attrs (g, 1);
+	    return;
+	}
 	if (aspect.badGraph) {
 	    agerr(AGWARN, "dot does not support the aspect attribute for disconnected graphs or graphs with clusters\n");
 	    asp = NULL;
 	    aspect.nextIter = 0;
 	}
-        dot_mincross(g,  (asp != NULL));
+        dot_mincross(g, (asp != NULL));
+	if (maxphase == 2) {
+	    attach_phase_attrs (g, 2);
+	    return;
+	}
         dot_position(g, asp);
+	if (maxphase == 3) {
+	    attach_phase_attrs (g, 2);  /* positions will be attached on output */
+	    return;
+	}
 	aspect.nPasses--;
     } while (aspect.nextIter && aspect.nPasses);
-
+#ifdef WITH_CGRAPH
+    if (GD_flags(g) & NEW_RANK)
+	removeFill (g);
+#endif
     dot_sameports(g);
     dot_splines(g);
     if (mapbool(agget(g, "compound")))
