@@ -1,4 +1,4 @@
-/* $Id: dotsplines.c,v 1.73 2011/02/06 00:19:45 erg Exp $ $Revision: 1.73 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -50,8 +50,12 @@
 #else /* WITH_CGRAPH */
 #define MAKEFWDEDGE(new, old) { \
 	edge_t *newp; \
+	Agedgeinfo_t *info; \
 	newp = new; \
+	info = (Agedgeinfo_t*)newp->base.data; \
+	*info = *(Agedgeinfo_t*)old->base.data; \
 	*newp = *old; \
+	newp->base.data = (Agrec_t*)info; \
 	AGTAIL(newp) = AGHEAD(old); \
 	AGHEAD(newp) = AGTAIL(old); \
 	ED_tail_port(newp) = ED_head_port(old); \
@@ -248,7 +252,7 @@ setEdgeLabelPos (graph_t * g)
 	    else if ((l = ND_label(n))) {// label of regular edge
 		place_vnlabel(n);
 	    }
-	    updateBB(g, l);
+	    if (l) updateBB(g, l);
 	}
     }
 }
@@ -264,17 +268,34 @@ static void _dot_splines(graph_t * g, int normalize)
 {
     int i, j, k, n_nodes, n_edges, ind, cnt;
     node_t *n;
+#ifndef WITH_CGRAPH
     edge_t fwdedgea, fwdedgeb;
+#else
+    Agedgeinfo_t fwdedgeai, fwdedgebi;
+    Agedgepair_t fwdedgea, fwdedgeb;
+#endif
     edge_t *e, *e0, *e1, *ea, *eb, *le0, *le1, **edges;
     path *P;
     spline_info_t sd;
-#ifndef WITH_CGRAPH
-    int et = EDGE_TYPE(g);
-#else /* WITH_CGRAPH */
     int et = EDGE_TYPE(g);
+#ifdef WITH_CGRAPH
+    fwdedgea.out.base.data = (Agrec_t*)&fwdedgeai;
+    fwdedgeb.out.base.data = (Agrec_t*)&fwdedgebi;
 #endif /* WITH_CGRAPH */
 
     if (et == ET_NONE) return; 
+    if (et == ET_CURVED) {
+	resetRW (g);
+	if (GD_has_labels(g) & EDGE_LABEL) {
+	    agerr (AGWARN, "edge labels with splines=curved not supported in dot - use xlabels\n");
+	}
+	for (n = agfstnode (g); n; n = agnxtnode(g, n)) {
+	    for (e = agfstout(g, n); e; e = agnxtout(g,e)) {
+		makeStraightEdge(g, e, et, &sinfo);
+	    }
+	}
+	goto finish;
+    } 
 #ifdef ORTHO
     if (et == ET_ORTHO) {
 	resetRW (g);
@@ -289,7 +310,7 @@ static void _dot_splines(graph_t * g, int normalize)
 #endif
 
     mark_lowclusters(g);
-    routesplinesinit();
+    if (routesplinesinit()) return;
     P = NEW(path);
     /* FlatHeight = 2 * GD_nodesep(g); */
     sd.Splinesep = GD_nodesep(g) / 4;
@@ -387,8 +408,13 @@ static void _dot_splines(graph_t * g, int normalize)
 	ea = (ED_tail_port(e0).defined
 	      || ED_head_port(e0).defined) ? e0 : le0;
 	if (ED_tree_index(ea) & BWDEDGE) {
+#ifndef WITH_CGRAPH
 	    MAKEFWDEDGE(&fwdedgea, ea);
 	    ea = &fwdedgea;
+#else
+	    MAKEFWDEDGE(&fwdedgea.out, ea);
+	    ea = &fwdedgea.out;
+#endif
 	}
 	for (cnt = 1; i < n_edges; cnt++, i++) {
 	    if (le0 != (le1 = getmainedge((e1 = edges[i]))))
@@ -397,8 +423,13 @@ static void _dot_splines(graph_t * g, int normalize)
 	    eb = (ED_tail_port(e1).defined
 		  || ED_head_port(e1).defined) ? e1 : le1;
 	    if (ED_tree_index(eb) & BWDEDGE) {
+#ifndef WITH_CGRAPH
 		MAKEFWDEDGE(&fwdedgeb, eb);
 		eb = &fwdedgeb;
+#else
+		MAKEFWDEDGE(&fwdedgeb.out, eb);
+		eb = &fwdedgeb.out;
+#endif
 	    }
 	    if (portcmp(ED_tail_port(ea), ED_tail_port(eb)))
 		break;
@@ -456,25 +487,31 @@ static void _dot_splines(graph_t * g, int normalize)
     if (normalize)
 	edge_normalize(g);
 
-#ifdef ORTHO
 finish :
-#endif
     /* vladimir: place port labels */
     /* FIX: head and tail labels are not part of cluster bbox */
-    if (E_headlabel || E_taillabel) {
+    if ((E_headlabel || E_taillabel) && (E_labelangle || E_labeldistance)) {
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    if (E_headlabel) {
 		for (e = agfstin(g, n); e; e = agnxtin(g, e))
+#ifndef WITH_CGRAPH
 		    if (ED_head_label(e)) {
-			place_portlabel(e, TRUE);
-			updateBB(g, ED_head_label(e));
+			if (place_portlabel(e, TRUE))
+			    updateBB(g, ED_head_label(e));
 		    }
+#else
+		    if (ED_head_label(AGMKOUT(e))) {
+			place_portlabel(AGMKOUT(e), TRUE);
+			updateBB(g, ED_head_label(AGMKOUT(e)));
+		    }
+#endif
+
 	    }
 	    if (E_taillabel) {
 		for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
 		    if (ED_tail_label(e)) {
-			place_portlabel(e, FALSE);
-			updateBB(g, ED_tail_label(e));
+			if (place_portlabel(e, FALSE))
+			    updateBB(g, ED_tail_label(e));
 		    }
 		}
 	    }
@@ -483,17 +520,18 @@ finish :
     /* end vladimir */
 
 #ifdef ORTHO
-    if (et != ET_ORTHO) {
+    if ((et != ET_ORTHO) && (et != ET_CURVED))  {
+#else
+    if (et != ET_CURVED) {
 #endif
 	free(edges);
 	free(P->boxes);
 	free(P);
 	free(sd.Rank_box);
 	routesplinesterm();
-#ifdef ORTHO
     } 
-#endif
     State = GVSPLINES;
+    EdgeLabelsDone = 1;
 }
 
 /* dot_splines:
@@ -568,10 +606,20 @@ setflags(edge_t *e, int hint1, int hint2, int f3)
  */
 static int edgecmp(edge_t** ptr0, edge_t** ptr1)
 {
+#ifndef WITH_CGRAPH
     edge_t fwdedgea, fwdedgeb, *e0, *e1, *ea, *eb, *le0, *le1;
+#else
+    Agedgeinfo_t fwdedgeai, fwdedgebi;
+    Agedgepair_t fwdedgea, fwdedgeb;
+    edge_t *e0, *e1, *ea, *eb, *le0, *le1;
+#endif
     int et0, et1, v0, v1, rv;
     double t0, t1;
 
+#ifdef WITH_CGRAPH
+    fwdedgea.out.base.data = (Agrec_t*)&fwdedgeai;
+    fwdedgeb.out.base.data = (Agrec_t*)&fwdedgebi;
+#endif
     e0 = (edge_t *) * ptr0;
     e1 = (edge_t *) * ptr1;
     et0 = ED_tree_index(e0) & EDGETYPEMASK;
@@ -597,18 +645,28 @@ static int edgecmp(edge_t** ptr0, edge_t** ptr1)
 	return (v0 - v1);
 
     /* This provides a cheap test for edges having the same set of endpoints.  */
-    if (AGID(le0) != AGID(le1))
-	return (AGID(le0) - AGID(le1));
+    if (AGSEQ(le0) != AGSEQ(le1))
+	return (AGSEQ(le0) - AGSEQ(le1));
 
     ea = (ED_tail_port(e0).defined || ED_head_port(e0).defined) ? e0 : le0;
     if (ED_tree_index(ea) & BWDEDGE) {
+#ifndef WITH_CGRAPH
 	MAKEFWDEDGE(&fwdedgea, ea);
 	ea = &fwdedgea;
+#else
+	MAKEFWDEDGE(&fwdedgea.out, ea);
+	ea = &fwdedgea.out;
+#endif
     }
     eb = (ED_tail_port(e1).defined || ED_head_port(e1).defined) ? e1 : le1;
     if (ED_tree_index(eb) & BWDEDGE) {
+#ifndef WITH_CGRAPH
 	MAKEFWDEDGE(&fwdedgeb, eb);
 	eb = &fwdedgeb;
+#else
+	MAKEFWDEDGE(&fwdedgeb.out, eb);
+	eb = &fwdedgeb.out;
+#endif
     }
     if ((rv = portcmp(ED_tail_port(ea), ED_tail_port(eb))))
 	return rv;
@@ -623,7 +681,7 @@ static int edgecmp(edge_t** ptr0, edge_t** ptr1)
     if (et0 == FLATEDGE && ED_label(e0) != ED_label(e1))
 	return (int) (ED_label(e0) - ED_label(e1));
 
-    return (AGID(e0) - AGID(e1));
+    return (AGSEQ(e0) - AGSEQ(e1));
 }
 
 /* cloneGraph:
@@ -656,14 +714,11 @@ cloneGraph (graph_t* g)
     auxg = agopen ("auxg", AG_IS_DIRECTED(g)?AGDIGRAPH:AGRAPH);
     agraphattr(auxg, "rank", "");
 #else /* WITH_CGRAPH */
-//    auxg = agopen ("auxg", AG_IS_DIRECTED(g)?AGDIGRAPH:AGRAPH);
-
-	if (agisdirected(g))
-		auxg = agopen ("auxg",Agdirected, NIL(Agdisc_t *));
-	else
-		auxg = agopen ("auxg",Agundirected, NIL(Agdisc_t *));
-
-
+    if (agisdirected(g))
+	auxg = agopen ("auxg",Agdirected, NIL(Agdisc_t *));
+    else
+	auxg = agopen ("auxg",Agundirected, NIL(Agdisc_t *));
+    agbindrec(auxg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
     agattr(auxg, AGRAPH, "rank", "");
 #endif /* WITH_CGRAPH */
     GD_drawing(auxg) = NEW(layout_t);
@@ -774,8 +829,7 @@ cloneNode (graph_t* g, node_t* orign, int flipped)
     node_t* n = agnode(g, orign->name);
 #else /* WITH_CGRAPH */
     node_t* n = agnode(g, agnameof(orign),1);
-    agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
-
+    agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
 #endif /* WITH_CGRAPH */
     agcopyattr (orign, n);
     if (shapeOf(orign) == SH_RECORD) {
@@ -897,6 +951,7 @@ makeSimpleFlatLabels (node_t* tn, node_t* hn, edge_t** edges, int ind, int cnt,
     ctrx = (leftend + rightend)/2.0;
     
     /* do first edge */
+    e = earray[0];
     pointn = 0;
     points[pointn++] = tp;
     points[pointn++] = tp;
@@ -917,7 +972,7 @@ makeSimpleFlatLabels (node_t* tn, node_t* hn, edge_t** edges, int ind, int cnt,
     umaxx = ctrx + (ED_label(e)->dimen.x)/2.0;
 
     for (i = 1; i < n_lbls; i++) {
-	e = edges[ind + i];
+	e = earray[i];
 	if (i%2) {  /* down */
 	    if (i == 1) {
 		lminx = ctrx - (ED_label(e)->dimen.x)/2.0;
@@ -975,7 +1030,7 @@ makeSimpleFlatLabels (node_t* tn, node_t* hn, edge_t** edges, int ind, int cnt,
 
     /* edges with no labels */
     for (; i < cnt; i++) {
-	e = edges[ind + i];
+	e = earray[i];
 	if (i%2) {  /* down */
 	    if (i == 1) {
 		lminx = (2*leftend + rightend)/3.0;
@@ -1124,8 +1179,7 @@ make_flat_adj_edges(path* P, edge_t** edges, int ind, int cnt, edge_t* e0,
     subg = agsubg (auxg, "xxx");
 #else /* WITH_CGRAPH */
     subg = agsubg (auxg, "xxx",1);
-	agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);	//node custom data
-
+    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
 #endif /* WITH_CGRAPH */
     agset (subg, "rank", "source");
     rightx = ND_coord(hn).x;
@@ -1439,21 +1493,46 @@ static void
 make_flat_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt, int et)
 {
     node_t *tn, *hn;
+#ifndef WITH_CGRAPH
     edge_t fwdedge, *e;
-    int j, i, r;
+#else
+    Agedgeinfo_t fwdedgei;
+    Agedgepair_t fwdedge;
+    edge_t *e;
+#endif
+    int j, i, r, isAdjacent;
     double stepx, stepy, vspace;
     int tside, hside, pn;
     pointf *ps;
     pathend_t tend, hend;
     graph_t* g;
 
+#ifdef WITH_CGRAPH
+    fwdedge.out.base.data = (Agrec_t*)&fwdedgei;
+#endif
+
     /* Get sample edge; normalize to go from left to right */
     e = edges[ind];
+    isAdjacent = ED_adjacent(e);
     if (ED_tree_index(e) & BWDEDGE) {
+#ifndef WITH_CGRAPH
 	MAKEFWDEDGE(&fwdedge, e);
 	e = &fwdedge;
+#else
+	MAKEFWDEDGE(&fwdedge.out, e);
+	e = &fwdedge.out;
+#endif
     }
-    if (ED_adjacent(edges[ind])) {
+    for (i = 1; i < cnt; i++) {
+	if (ED_adjacent(edges[ind+i])) {
+	    isAdjacent = 1;
+	    break;
+	}
+    }
+    /* The lead edge edges[ind] might not have been marked earlier as adjacent,
+     * so check them all.
+     */
+    if (isAdjacent) {
 	make_flat_adj_edges (P, edges, ind, cnt, e, et);
 	return;
     }
@@ -1634,7 +1713,13 @@ make_regular_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt
 {
     graph_t *g;
     node_t *tn, *hn;
+#ifndef WITH_CGRAPH
     edge_t fwdedgea, fwdedgeb, fwdedge, *e, *fe, *le, *segfirst;
+#else
+    Agedgeinfo_t fwdedgeai, fwdedgebi, fwdedgei;
+    Agedgepair_t fwdedgea, fwdedgeb, fwdedge;
+    edge_t *e, *fe, *le, *segfirst;
+#endif /* WITH_CGRAPH */
     pointf *ps;
     pathend_t tend, hend;
     boxf b;
@@ -1645,6 +1730,12 @@ make_regular_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt
     static int numpts2;
     int pointn;
 
+#ifdef WITH_CGRAPH
+    fwdedgea.out.base.data = (Agrec_t*)&fwdedgeai;
+    fwdedgeb.out.base.data = (Agrec_t*)&fwdedgebi;
+    fwdedge.out.base.data = (Agrec_t*)&fwdedgei;
+#endif /* WITH_CGRAPH */
+
     if (!pointfs) {
 	pointfs = N_GNEW(NUMPTS, pointf);
    	pointfs2 = N_GNEW(NUMPTS, pointf);
@@ -1656,22 +1747,32 @@ make_regular_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt
     g = agraphof(agtail(e));
     hackflag = FALSE;
     if (ABS(ND_rank(agtail(e)) - ND_rank(aghead(e))) > 1) {
-	fwdedgea = *e;
-	if (ED_tree_index(e) & BWDEDGE) {
-	    MAKEFWDEDGE(&fwdedgeb, e);
 #ifndef WITH_CGRAPH
+	fwdedgea = *e;
+#else /* WITH_CGRAPH */
+	fwdedgeai = *(Agedgeinfo_t*)e->base.data;
+	fwdedgea.out = *e;
+	fwdedgea.out.base.data = (Agrec_t*)&fwdedgeai;
+#endif /* WITH_CGRAPH */
+	if (ED_tree_index(e) & BWDEDGE) {
+#ifndef WITH_CGRAPH
+	    MAKEFWDEDGE(&fwdedgeb, e);
 	    fwdedgea.tail = aghead(e);
 	    fwdedgea.u.tail_port = ED_head_port(e);
 #else /* WITH_CGRAPH */
-	    agtail(&fwdedgea) = aghead(e);
-	    ED_tail_port(&fwdedgea) = ED_head_port(e);
+	    MAKEFWDEDGE(&fwdedgeb.out, e);
+	    agtail(&fwdedgea.out) = aghead(e);
+	    ED_tail_port(&fwdedgea.out) = ED_head_port(e);
 #endif /* WITH_CGRAPH */
 	} else {
-	    fwdedgeb = *e;
 #ifndef WITH_CGRAPH
+	    fwdedgeb = *e;
 	    fwdedgea.tail = e->tail;
 #else /* WITH_CGRAPH */
-	    agtail(&fwdedgea) = agtail(e);
+	    fwdedgebi = *(Agedgeinfo_t*)e->base.data;
+	    fwdedgeb.out = *e;
+	    fwdedgeb.out.base.data = (Agrec_t*)&fwdedgebi;
+	    agtail(&fwdedgea.out) = agtail(e);
 #endif /* WITH_CGRAPH */
 	}
 	le = getmainedge(e);
@@ -1683,19 +1784,25 @@ make_regular_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt
 	fwdedgea.u.edge_type = VIRTUAL;
 	fwdedgea.u.head_port.p.x = fwdedgea.u.head_port.p.y = 0;
 	fwdedgea.u.to_orig = e;
-#else /* WITH_CGRAPH */
-	aghead(&fwdedgea) = aghead(le);
-	ED_head_port(&fwdedgea).defined = FALSE;
-	ED_edge_type(&fwdedgea) = VIRTUAL;
-	ED_head_port(&fwdedgea).p.x = ED_head_port(&fwdedgea).p.y = 0;
-	ED_to_orig(&fwdedgea) = e;
-#endif /* WITH_CGRAPH */
 	e = &fwdedgea;
+#else /* WITH_CGRAPH */
+	aghead(&fwdedgea.out) = aghead(le);
+	ED_head_port(&fwdedgea.out).defined = FALSE;
+	ED_edge_type(&fwdedgea.out) = VIRTUAL;
+	ED_head_port(&fwdedgea.out).p.x = ED_head_port(&fwdedgea.out).p.y = 0;
+	ED_to_orig(&fwdedgea.out) = e;
+	e = &fwdedgea.out;
+#endif /* WITH_CGRAPH */
 	hackflag = TRUE;
     } else {
 	if (ED_tree_index(e) & BWDEDGE) {
+#ifndef WITH_CGRAPH
 	    MAKEFWDEDGE(&fwdedgea, e);
 	    e = &fwdedgea;
+#else
+	    MAKEFWDEDGE(&fwdedgea.out, e);
+	    e = &fwdedgea.out;
+#endif
 	}
     }
     fe = e;
@@ -1785,8 +1892,11 @@ make_regular_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt
 	}
 	boxes[boxn++] = rank_box(sp, g, ND_rank(tn));
 	b = hend.nb = maximal_bbox(sp, hn, e, NULL);
-	endpath(P, hackflag ? &fwdedgeb : e, REGULAREDGE, &hend,
-	        spline_merge(aghead(e)));
+#ifndef WITH_CGRAPH
+	endpath(P, hackflag ? &fwdedgeb : e, REGULAREDGE, &hend, spline_merge(aghead(e)));
+#else
+	endpath(P, hackflag ? &fwdedgeb.out : e, REGULAREDGE, &hend, spline_merge(aghead(e)));
+#endif
 	b.UR.y = hend.boxes[hend.boxn - 1].UR.y;
 	b.LL.y = hend.boxes[hend.boxn - 1].LL.y;
 	b = makeregularend(b, TOP,
@@ -1816,7 +1926,11 @@ make_regular_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt
 	    pointfs[pointn++] = ps[i];
 	}
 	recover_slack(segfirst, P);
+#ifndef WITH_CGRAPH
 	hn = hackflag ? aghead(&fwdedgeb) : aghead(e);
+#else
+	hn = hackflag ? aghead(&fwdedgeb.out) : aghead(e);
+#endif
     }
 
     /* make copies of the spline points, one per multi-edge */
@@ -1839,8 +1953,13 @@ make_regular_edge(spline_info_t* sp, path * P, edge_t ** edges, int ind, int cnt
     for (j = 1; j < cnt; j++) {
 	e = edges[ind + j];
 	if (ED_tree_index(e) & BWDEDGE) {
+#ifndef WITH_CGRAPH
 	    MAKEFWDEDGE(&fwdedge, e);
 	    e = &fwdedge;
+#else
+	    MAKEFWDEDGE(&fwdedge.out, e);
+	    e = &fwdedge.out;
+#endif
 	}
 	for (i = 1; i < pointn - 1; i++)
 	    pointfs[i].x += sp->Multisep;
@@ -1869,24 +1988,24 @@ completeregularpath(path * P, edge_t * first, edge_t * last,
     uleft = uright = NULL;
     uleft = top_bound(first, -1), uright = top_bound(first, 1);
     if (uleft) {
-	spl = getsplinepoints(uleft);
+	if (!(spl = getsplinepoints(uleft))) return;
 	pp = spl->list[0].list;
        	pn = spl->list[0].size;
     }
     if (uright) {
-	spl = getsplinepoints(uright);
+	if (!(spl = getsplinepoints(uright))) return;
 	pp = spl->list[0].list;
        	pn = spl->list[0].size;
     }
     lleft = lright = NULL;
     lleft = bot_bound(last, -1), lright = bot_bound(last, 1);
     if (lleft) {
-	spl = getsplinepoints(lleft);
+	if (!(spl = getsplinepoints(lleft))) return;
 	pp = spl->list[spl->size - 1].list;
        	pn = spl->list[spl->size - 1].size;
     }
     if (lright) {
-	spl = getsplinepoints(lright);
+	if (!(spl = getsplinepoints(lright))) return;
 	pp = spl->list[spl->size - 1].list;
        	pn = spl->list[spl->size - 1].size;
     }
@@ -2044,7 +2163,7 @@ int *boxnp;
 	j += nsub - 1;
     }
     if (left) {
-	lspls = getsplinepoints(left);
+	if (!(lspls = getsplinepoints(left))) return;
 	pp = spline_at_y(lspls, boxes[0].UR.y);
 	for (i = 0; i < nsub; i++) {
 	    cp = spline_at_y(lspls, boxes[i].LL.y);
@@ -2065,7 +2184,7 @@ int *boxnp;
 	    boxes[i].LL.x = endp->boxes[endp->boxn - 1].UR.x - MINW;
     }
     if (right) {
-	rspls = getsplinepoints(right);
+	if (!(rspls = getsplinepoints(right))) return;
 	pp = spline_at_y(rspls, boxes[0].UR.y);
 	for (i = 0; i < nsub; i++) {
 	    cp = spline_at_y(rspls, boxes[i].LL.y);
@@ -2279,6 +2398,13 @@ static int cl_vninside(graph_t * cl, node_t * n)
 	    BETWEEN(GD_bb(cl).LL.y, (double)(ND_coord(n).y), GD_bb(cl).UR.y));
 }
 
+/* All nodes belong to some cluster, which may be the root graph.
+ * For the following, we only want a cluster if it is a real cluster
+ * It is not clear this will handle all potential problems. It seems one
+ * could have hcl and tcl contained in cl, which would also cause problems.
+ */
+#define REAL_CLUSTER(n) (ND_clust(n)==agraphof(n)?NULL:ND_clust(n))
+
 /* returns the cluster of (adj) that interferes with n,
  */
 static Agraph_t *cl_bound(n, adj)
@@ -2296,16 +2422,16 @@ node_t *n, *adj;
 	hcl = ND_clust(aghead(orig));
     }
     if (ND_node_type(adj) == NORMAL) {
-	cl = ND_clust(adj);
+	cl = REAL_CLUSTER(adj);
 	if (cl && (cl != tcl) && (cl != hcl))
 	    rv = cl;
     } else {
 	orig = ED_to_orig(ND_out(adj).list[0]);
-	cl = ND_clust(agtail(orig));
+	cl = REAL_CLUSTER(agtail(orig));
 	if (cl && (cl != tcl) && (cl != hcl) && cl_vninside(cl, adj))
 	    rv = cl;
 	else {
-	    cl = ND_clust(aghead(orig));
+	    cl = REAL_CLUSTER(aghead(orig));
 	    if (cl && (cl != tcl) && (cl != hcl) && cl_vninside(cl, adj))
 		rv = cl;
 	}
@@ -2371,8 +2497,10 @@ static boxf maximal_bbox(spline_info_t* sp, node_t* vn, edge_t* ie, edge_t* oe)
     } else
 	rv.UR.x = MAX(ROUND(b), sp->RightBound);
 
-    if ((ND_node_type(vn) == VIRTUAL) && (ND_label(vn)))
+    if ((ND_node_type(vn) == VIRTUAL) && (ND_label(vn))) {
 	rv.UR.x -= ND_rw(vn);
+	if (rv.UR.x < rv.LL.x) rv.UR.x = ND_coord(vn).x;
+    }
 
     rv.LL.y = ND_coord(vn).y - GD_rank(g)[ND_rank(vn)].ht1;
     rv.UR.y = ND_coord(vn).y + GD_rank(g)[ND_rank(vn)].ht2;
@@ -2456,14 +2584,14 @@ edge_t *ie1, *oe1;
 void showpath(path * p)
 {
     int i;
-    point LL, UR;
+    pointf LL, UR;
 
     fprintf(stderr, "%%!PS\n");
     for (i = 0; i < p->nbox; i++) {
 	LL = p->boxes[i].LL;
 	UR = p->boxes[i].UR;
 	fprintf(stderr,
-		"newpath %d %d moveto %d %d lineto %d %d lineto %d %d lineto closepath stroke\n",
+		"newpath %.04f %.04f moveto %.04f %.04f lineto %.04f %.04f lineto %.04f %.04f lineto closepath stroke\n",
 		LL.x, LL.y, UR.x, LL.y, UR.x, UR.y, LL.x, UR.y);
     }
     fprintf(stderr, "showpage\n");
