@@ -1,4 +1,4 @@
-/* $Id: fastgr.c,v 1.8 2011/01/25 16:30:48 ellson Exp $ $Revision: 1.8 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -166,15 +166,21 @@ edge_t *new_virtual_edge(node_t * u, node_t * v, edge_t * orig)
 #ifndef WITH_CGRAPH
     e = NEW(edge_t);
 #else /* WITH_CGRAPH */
-    e=agedge(agraphof(orig),u,v,"",1);
-    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//graph custom data
+    Agedgepair_t* e2 = NEW(Agedgepair_t);
+    AGTYPE(&(e2->in)) = AGINEDGE;
+    AGTYPE(&(e2->out)) = AGOUTEDGE;
+    e2->out.base.data = (Agrec_t*)NEW(Agedgeinfo_t);
+    e = &(e2->out);
 #endif /* WITH_CGRAPH */
     agtail(e) = u;
     aghead(e) = v;
     ED_edge_type(e) = VIRTUAL;
 
     if (orig) {
-	AGID(e) = AGID(orig);
+	AGSEQ(e) = AGSEQ(orig);
+#ifdef WITH_CGRAPH
+	AGSEQ(&(e2->in)) = AGSEQ(orig);
+#endif
 	ED_count(e) = ED_count(orig);
 	ED_xpenalty(e) = ED_xpenalty(orig);
 	ED_weight(e) = ED_weight(orig);
@@ -247,6 +253,8 @@ node_t *virtual_node(graph_t * g)
     n->graph = g;
 #else /* WITH_CGRAPH */
 //  agnameof(n) = "virtual";
+    AGTYPE(n) = AGNODE;
+    n->base.data = (Agrec_t*)NEW(Agnodeinfo_t);
     n->root = g;
 #endif /* WITH_CGRAPH */
     ND_node_type(n) = VIRTUAL;
@@ -282,7 +290,7 @@ static char *NAME(node_t * n)
     static char buf[20];
     if (ND_node_type(n) == NORMAL)
 	return n->name;
-    sprintf(buf, "V%x", n);
+    sprintf(buf, "V%p", n);
     return buf;
 }
 
