@@ -1,4 +1,4 @@
-/* $Id: mincross.c,v 1.23 2011/02/04 18:36:12 erg Exp $ $Revision: 1.23 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -39,7 +39,6 @@ static void init_mccomp(graph_t * g, int c);
 static void cleanup2(graph_t * g, int nc);
 static int mincross_clust(graph_t * par, graph_t * g, int);
 static int mincross(graph_t * g, int startpass, int endpass, int);
-static void init_mincross(graph_t * g);
 static void mincross_step(graph_t * g, int pass);
 static void mincross_options(graph_t * g);
 static void save_best(graph_t * g);
@@ -742,6 +741,9 @@ static void cleanup2(graph_t * g, int nc)
 		for (j = 0; (e = ND_flat_out(v).list[j]); j++)
 		    if (ED_edge_type(e) == FLATORDER) {
 			delete_flat_edge(e);
+#ifdef WITH_CGRAPH
+			free(e->base.data);
+#endif
 			free(e);
 			j--;
 		    }
@@ -854,6 +856,69 @@ void rec_reset_vlists(graph_t * g)
 	}
 }
 
+#ifdef WITH_CGRAPH
+/* realFillRanks:
+ * The structures in crossing minimization and positioning require
+ * that clusters have some node on each rank. This function recursively
+ * guarantees this property. It takes into account nodes and edges in
+ * a cluster, the latter causing dummy nodes for intervening ranks.
+ * For any rank without node, we create a real node of small size. This
+ * is stored in the subgraph sg, for easy removal later.
+ *
+ * I believe it is not necessary to do this for the root graph, as these
+ * are laid out one component at a time and these will necessarily have a
+ * node on each rank from source to sink levels.
+ */
+static Agraph_t*
+realFillRanks (Agraph_t* g, int rnks[], int rnks_sz, Agraph_t* sg)
+{
+    int i, c;
+    Agedge_t* e;
+    Agnode_t* n;
+
+    for (c = 1; c <= GD_n_cluster(g); c++)
+	sg = realFillRanks (GD_clust(g)[c], rnks, rnks_sz, sg);
+
+    if (agroot(g) == g)
+	return sg;
+    memset (rnks, 0, sizeof(int)*rnks_sz);
+    for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+	rnks[ND_rank(n)] = 1;
+	for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
+	    for (i = ND_rank(n)+1; i <= ND_rank(aghead(e)); i++) 
+		rnks[i] = 1;
+	}
+    }
+    for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
+	if (rnks[i] == 0) {
+	    if (!sg) {
+		sg = agsubg (agroot(g), "_new_rank", 1);
+	    }
+	    n = agnode (sg, NULL, 1);
+	    agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+	    ND_rank(n) = i;
+	    ND_lw(n) = ND_rw(n) = 0.5;
+	    ND_ht(n) = 1;
+	    ND_UF_size(n) = 1;
+	    alloc_elist(4, ND_in(n));
+	    alloc_elist(4, ND_out(n));
+	    agsubnode (g, n, 1);
+	}
+    }
+    return sg;
+}
+
+static void
+fillRanks (Agraph_t* g)
+{
+    Agraph_t* sg;
+    int rnks_sz = GD_maxrank(g) + 2;
+    int* rnks = N_NEW(rnks_sz, int);
+    sg = realFillRanks (g, rnks, rnks_sz, NULL);
+    free (rnks);
+}
+#endif
+
 static void init_mincross(graph_t * g)
 {
     int size;
@@ -870,6 +935,10 @@ static void init_mincross(graph_t * g)
     TE_list = N_NEW(size, edge_t *);
     TI_list = N_NEW(size, int);
     mincross_options(g);
+#ifdef WITH_CGRAPH
+    if (GD_flags(g) & NEW_RANK)
+	fillRanks (g);
+#endif
     class2(g);
     decompose(g, 1);
     allocate_ranks(g);
@@ -1017,9 +1086,9 @@ void install_in_rank(graph_t * g, node_t * n)
     r = ND_rank(n);
     i = GD_rank(g)[r].n;
     if (GD_rank(g)[r].an <= 0) {
-	agerr(AGERR, "install_in_rank %s %s rank %d i = %d an = 0\n",
-	      agnameof(g), agnameof(n), r, i);
-	abort();
+	agerr(AGERR, "install_in_rank, line %d: %s %s rank %d i = %d an = 0\n",
+	      __LINE__, agnameof(g), agnameof(n), r, i);
+	return;
     }
 
     GD_rank(g)[r].v[i] = n;
@@ -1036,13 +1105,22 @@ void install_in_rank(graph_t * g, node_t * n)
 	assert(v != NULL);
     }
 #endif
-    if (ND_order(n) > GD_rank(Root)[r].an)
-	abort();
-    if ((r < GD_minrank(g)) || (r > GD_maxrank(g)))
-	abort();
+    if (ND_order(n) > GD_rank(Root)[r].an) {
+	agerr(AGERR, "install_in_rank, line %d: ND_order(%s) [%d] > GD_rank(Root)[%d].an [%d]\n",
+	      __LINE__, agnameof(n), ND_order(n), r, GD_rank(Root)[r].an);
+	return;
+    }
+    if ((r < GD_minrank(g)) || (r > GD_maxrank(g))) {
+	agerr(AGERR, "install_in_rank, line %d: rank %d not in rank range [%d,%d]\n",
+	      __LINE__, r, GD_minrank(g), GD_maxrank(g));
+	return;
+    }
     if (GD_rank(g)[r].v + ND_order(n) >
-	GD_rank(g)[r].av + GD_rank(Root)[r].an)
-	abort();
+	GD_rank(g)[r].av + GD_rank(Root)[r].an) {
+	agerr(AGERR, "install_in_rank, line %d: GD_rank(g)[%d].v + ND_order(%s) [%d] > GD_rank(g)[%d].av + GD_rank(Root)[%d].an [%d]\n",
+	      __LINE__, r, agnameof(n),GD_rank(g)[r].v + ND_order(n), r, r, GD_rank(g)[r].av+GD_rank(Root)[r].an);
+	return;
+    }
 }
 
 /*	install nodes in ranks. the initial ordering ensure that series-parallel
@@ -1534,7 +1612,7 @@ static int nodeposcmpf(node_t ** n0, node_t ** n1)
 
 static int edgeidcmpf(edge_t ** e0, edge_t ** e1)
 {
-    return (AGID(*e0) - AGID(*e1));
+    return (AGSEQ(*e0) - AGSEQ(*e1));
 }
 
 /* following code deals with weights of edges of "virtual" nodes */
