@@ -1,18 +1,15 @@
-/* $Id: position.c,v 1.32 2009/11/30 19:21:23 erg Exp $ $Revision: 1.32 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 /*
@@ -58,11 +55,11 @@ dumpNS (graph_t * g)
 }
 #endif
 
-static void
+static double
 largeMinlen (double l)
 {
     agerr (AGERR, "Edge length %f larger than maximum %u allowed.\nCheck for overwide node(s).\n", l, USHRT_MAX); 
-    exit (1);
+    return (double)USHRT_MAX;
 }
 
 /* connectGraph:
@@ -175,11 +172,20 @@ edge_t *make_aux_edge(node_t * u, node_t * v, double len, int wt)
 {
     edge_t *e;
 
+#ifndef WITH_CGRAPH
     e = NEW(edge_t);
+#else
+    Agedgepair_t* e2 = NEW(Agedgepair_t);
+    AGTYPE(&(e2->in)) = AGINEDGE;
+    AGTYPE(&(e2->out)) = AGOUTEDGE;
+    e2->out.base.data = (Agrec_t*)NEW(Agedgeinfo_t);
+    e = &(e2->out);
+#endif /* WITH_CGRAPH */
+
     agtail(e) = u;
     aghead(e) = v;
     if (len > USHRT_MAX)
-	largeMinlen (len);
+	len = largeMinlen (len);
     ED_minlen(e) = ROUND(len);
     ED_weight(e) = wt;
     fast_edge(e);
@@ -300,7 +306,7 @@ make_LR_constraints(graph_t * g)
                      */
 		    m0 = MAX(m0, width + GD_nodesep(g) + ROUND(ED_dist(e)));
 		    if (m0 > USHRT_MAX)
-			largeMinlen (m0);
+			m0 = largeMinlen (m0);
 		    ED_minlen(e0) = MAX(ED_minlen(e0), m0);
 		}
 		else if (!ED_label(e)) {
@@ -389,7 +395,7 @@ static int vnode_not_related_to(graph_t * g, node_t * v)
 /* keepout_othernodes:
  * Guarantee nodes outside the cluster g are placed outside of it.
  * This is done by adding constraints to make sure such nodes have
- * a gap of CL_OFFSET from the left or right bounding box node ln or rn.
+ * a gap of margin from the left or right bounding box node ln or rn.
  * 
  * We could probably reduce some of these constraints by checking if
  * the node is in a cluster, since elsewhere we make constrain a
@@ -398,9 +404,10 @@ static int vnode_not_related_to(graph_t * g, node_t * v)
  */
 static void keepout_othernodes(graph_t * g)
 {
-    int i, c, r;
+    int i, c, r, margin;
     node_t *u, *v;
 
+    margin = late_int (g, G_margin, CL_OFFSET, 0);
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	if (GD_rank(g)[r].n == 0)
 	    continue;
@@ -411,7 +418,7 @@ static void keepout_othernodes(graph_t * g)
 	    u = GD_rank(agroot(g))[r].v[i];
 	    /* can't use "is_a_vnode_of" because elists are swapped */
 	    if ((ND_node_type(u) == NORMAL) || vnode_not_related_to(g, u)) {
-		make_aux_edge(u, GD_ln(g), CL_OFFSET + ND_rw(u), 0);
+		make_aux_edge(u, GD_ln(g), margin + ND_rw(u), 0);
 		break;
 	    }
 	}
@@ -419,7 +426,7 @@ static void keepout_othernodes(graph_t * g)
 	     i++) {
 	    u = GD_rank(agroot(g))[r].v[i];
 	    if ((ND_node_type(u) == NORMAL) || vnode_not_related_to(g, u)) {
-		make_aux_edge(GD_rn(g), u, CL_OFFSET + ND_lw(u), 0);
+		make_aux_edge(GD_rn(g), u, margin + ND_lw(u), 0);
 		break;
 	    }
 	}
@@ -437,17 +444,18 @@ static void keepout_othernodes(graph_t * g)
  */
 static void contain_subclust(graph_t * g)
 {
-    int c;
+    int margin, c;
     graph_t *subg;
 
+    margin = late_int (g, G_margin, CL_OFFSET, 0);
     make_lrvn(g);
     for (c = 1; c <= GD_n_cluster(g); c++) {
 	subg = GD_clust(g)[c];
 	make_lrvn(subg);
 	make_aux_edge(GD_ln(g), GD_ln(subg),
-		      CL_OFFSET + GD_border(g)[LEFT_IX].x, 0);
+		      margin + GD_border(g)[LEFT_IX].x, 0);
 	make_aux_edge(GD_rn(subg), GD_rn(g),
-		      CL_OFFSET + GD_border(g)[RIGHT_IX].x, 0);
+		      margin + GD_border(g)[RIGHT_IX].x, 0);
 	contain_subclust(subg);
     }
 }
@@ -460,10 +468,11 @@ static void contain_subclust(graph_t * g)
  */
 static void separate_subclust(graph_t * g)
 {
-    int i, j;
+    int i, j, margin;
     graph_t *low, *high;
     graph_t *left, *right;
 
+    margin = late_int (g, G_margin, CL_OFFSET, 0);
     for (i = 1; i <= GD_n_cluster(g); i++)
 	make_lrvn(GD_clust(g)[i]);
     for (i = 1; i <= GD_n_cluster(g); i++) {
@@ -485,7 +494,7 @@ static void separate_subclust(graph_t * g)
 		left = high;
 		right = low;
 	    }
-	    make_aux_edge(GD_rn(left), GD_ln(right), CL_OFFSET, 0);
+	    make_aux_edge(GD_rn(left), GD_ln(right), margin, 0);
 	}
 	separate_subclust(GD_clust(g)[i]);
     }
@@ -546,8 +555,12 @@ static void remove_aux_edges(graph_t * g)
     edge_t *e;
 
     for (n = GD_nlist(g); n; n = ND_next(n)) {
-	for (i = 0; (e = ND_out(n).list[i]); i++)
+	for (i = 0; (e = ND_out(n).list[i]); i++) {
+#ifdef WITH_CGRAPH
+	    free(e->base.data);
+#endif
 	    free(e);
+	}
 	free_list(ND_out(n));
 	free_list(ND_in(n));
 	ND_out(n) = ND_save_out(n);
@@ -562,6 +575,9 @@ static void remove_aux_edges(graph_t * g)
 		ND_next(nprev) = nnext;
 	    else
 		GD_nlist(g) = nnext;
+#ifdef WITH_CGRAPH
+	    free(n->base.data);
+#endif
 	    free(n);
 	} else
 	    nprev = n;
@@ -696,9 +712,11 @@ static void adjustRanks(graph_t * g, int equal)
 {
     int lht;			/* label height */
     int rht;			/* height between top and bottom ranks */
-    int delta, maxr, minr;
+    int delta, maxr, minr, margin;
     int c, ht1, ht2;
+
     rank_t *rank = GD_rank(agroot(g));
+    margin = late_int (g, G_margin, CL_OFFSET, 0);
 
     ht1 = GD_ht1(g);
     ht2 = GD_ht2(g);
@@ -707,9 +725,9 @@ static void adjustRanks(graph_t * g, int equal)
 	graph_t *subg = GD_clust(g)[c];
 	adjustRanks(subg, equal);
 	if (GD_maxrank(subg) == GD_maxrank(g))
-	    ht1 = MAX(ht1, GD_ht1(subg) + CL_OFFSET);
+	    ht1 = MAX(ht1, GD_ht1(subg) + margin);
 	if (GD_minrank(subg) == GD_minrank(g))
-	    ht2 = MAX(ht2, GD_ht2(subg) + CL_OFFSET);
+	    ht2 = MAX(ht2, GD_ht2(subg) + margin);
     }
 
     GD_ht1(g) = ht1;
@@ -747,7 +765,12 @@ static int clust_ht(Agraph_t * g)
     int c, ht1, ht2;
     graph_t *subg;
     rank_t *rank = GD_rank(agroot(g));
-    int haveClustLabel = 0;
+    int margin, haveClustLabel = 0;
+
+    if (g == agroot(g)) 
+	margin = CL_OFFSET;
+    else
+	margin = late_int (g, G_margin, CL_OFFSET, 0);
 
     ht1 = GD_ht1(g);
     ht2 = GD_ht2(g);
@@ -757,9 +780,9 @@ static int clust_ht(Agraph_t * g)
 	subg = GD_clust(g)[c];
 	haveClustLabel |= clust_ht(subg);
 	if (GD_maxrank(subg) == GD_maxrank(g))
-	    ht1 = MAX(ht1, GD_ht1(subg) + CL_OFFSET);
+	    ht1 = MAX(ht1, GD_ht1(subg) + margin);
 	if (GD_minrank(subg) == GD_minrank(g))
-	    ht2 = MAX(ht2, GD_ht2(subg) + CL_OFFSET);
+	    ht2 = MAX(ht2, GD_ht2(subg) + margin);
     }
 
     /* account for a possible cluster label in clusters */
@@ -821,7 +844,7 @@ static void set_ycoords(graph_t * g)
 
 	    /* update nearest enclosing cluster rank ht */
 	    if ((clust = ND_clust(n))) {
-		int yoff = (clust == g ? 0 : CL_OFFSET);
+		int yoff = (clust == g ? 0 : late_int (clust, G_margin, CL_OFFSET, 0));
 		if (ND_rank(n) == GD_minrank(clust))
 		    GD_ht2(clust) = MAX(GD_ht2(clust), ht2 + yoff);
 		if (ND_rank(n) == GD_maxrank(clust))
@@ -1122,10 +1145,15 @@ static void do_leaves(graph_t * g, node_t * leader)
 	n = aghead(ND_out(leader).list[0]);
 	j = ND_order(leader) + 1;
 	for (e = agfstin(g, n); e; e = agnxtin(g, e)) {
-	    if ((agtail(e) != leader) && (UF_find(agtail(e)) == leader)) {
-		lbound = place_leaf(agtail(e), lbound, j++);
-		unmerge_oneway(e);
-		elist_append(e, ND_in(aghead(e)));
+#ifndef WITH_CGRAPH
+	    edge_t *e1 = e;
+#else
+	    edge_t *e1 = AGMKOUT(e);
+#endif
+	    if ((agtail(e1) != leader) && (UF_find(agtail(e1)) == leader)) {
+		lbound = place_leaf(agtail(e1), lbound, j++);
+		unmerge_oneway(e1);
+		elist_append(e1, ND_in(aghead(e1)));
 	    }
 	}
     } else {			/* out edge leaves */
@@ -1221,9 +1249,10 @@ static void make_lrvn(graph_t * g)
  */
 static void contain_nodes(graph_t * g)
 {
-    int r;
+    int margin, r;
     node_t *ln, *rn, *v;
 
+    margin = late_int (g, G_margin, CL_OFFSET, 0);
     make_lrvn(g);
     ln = GD_ln(g);
     rn = GD_rn(g);
@@ -1237,10 +1266,10 @@ static void contain_nodes(graph_t * g)
 	    continue;
 	}
 	make_aux_edge(ln, v,
-		      ND_lw(v) + CL_OFFSET + GD_border(g)[LEFT_IX].x, 0);
+		      ND_lw(v) + margin + GD_border(g)[LEFT_IX].x, 0);
 	v = GD_rank(g)[r].v[GD_rank(g)[r].n - 1];
 	make_aux_edge(v, rn,
-		      ND_rw(v) + CL_OFFSET + GD_border(g)[RIGHT_IX].x, 0);
+		      ND_rw(v) + margin + GD_border(g)[RIGHT_IX].x, 0);
     }
 }
 
