@@ -1,18 +1,15 @@
-/* $Id: constraint.c,v 1.19 2009/12/17 21:59:31 erg Exp $ $Revision: 1.19 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
-/**********************************************************
-*      This software is part of the graphviz package      *
-*                http://www.graphviz.org/                 *
-*                                                         *
-*            Copyright (c) 1994-2004 AT&T Corp.           *
-*                and is licensed under the                *
-*            Common Public License, Version 1.0           *
-*                      by AT&T Corp.                      *
-*                                                         *
-*        Information and Software Systems Research        *
-*              AT&T Research, Florham Park NJ             *
-**********************************************************/
+/*************************************************************************
+ * Copyright (c) 2011 AT&T Intellectual Property 
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ *************************************************************************/
 
 
 #ifdef HAVE_CONFIG_H
@@ -147,6 +144,7 @@ static void mapGraphs(graph_t * g, graph_t * cg, distfn dist)
 	    ce = agedge(cg, t, h);
 #else
 	    ce = agedge(cg, t, h, NULL, 1);
+	    agbindrec(ce, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
 #endif
 	    ED_weight(ce) = 1;
 	    if (ED_minlen(ce) < delta) {
@@ -229,14 +227,15 @@ static graph_t *mkNConstraintG(graph_t * g, Dt_t * list,
 {
     nitem *p;
     nitem *nxp;
+    node_t *n;
+    edge_t *e;
+    node_t *lastn = NULL;
 #ifndef WITH_CGRAPH
     graph_t *cg = agopen("cg", AGDIGRAPHSTRICT);
 #else
     graph_t *cg = agopen("cg", Agstrictdirected, NIL(Agdisc_t *));
+    agbindrec(cg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);  // graph custom data
 #endif
-    node_t *n;
-    edge_t *e;
-    node_t *lastn = NULL;
 
     for (p = (nitem *) dtflatten(list); p;
 	 p = (nitem *) dtlink(list, (Dtlink_t *) p)) {
@@ -268,6 +267,7 @@ static graph_t *mkNConstraintG(graph_t * g, Dt_t * list,
 	        e = agedge(cg, p->cnode, nxp->cnode);
 #else
 	        e = agedge(cg, p->cnode, nxp->cnode, NULL, 1);
+		agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);   // edge custom data
 #endif
 		assert (delta <= 0xFFFF);
 		ED_minlen(e) = delta;
@@ -316,11 +316,6 @@ static graph_t *mkConstraintG(graph_t * g, Dt_t * list,
     nitem *p;
     nitem *nxt = NULL;
     nitem *nxp;
-#ifndef WITH_CGRAPH
-    graph_t *cg = agopen("cg", AGDIGRAPHSTRICT);
-#else
-    graph_t *cg = agopen("cg", Agstrictdirected, NIL(Agdisc_t *));
-#endif
     graph_t *vg;
     node_t *prev = NULL;
     node_t *root = NULL;
@@ -332,6 +327,12 @@ static graph_t *mkConstraintG(graph_t * g, Dt_t * list,
     double root_val;
 #endif
     node_t *lastn = NULL;
+#ifndef WITH_CGRAPH
+    graph_t *cg = agopen("cg", AGDIGRAPHSTRICT);
+#else
+    graph_t *cg = agopen("cg", Agstrictdirected, NIL(Agdisc_t *));
+    agbindrec(cg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);  // graph custom data
+#endif
 
     /* count distinct nodes */
     cnt = 0;
@@ -378,6 +379,7 @@ static graph_t *mkConstraintG(graph_t * g, Dt_t * list,
 		e = agedge(cg, prev, n);
 #else
 		e = agedge(cg, prev, n, NULL, 1);
+		agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);   // edge custom data
 #endif
 		ED_minlen(e) = SCALE;
 		ED_weight(e) = 1;
@@ -406,9 +408,8 @@ static graph_t *mkConstraintG(graph_t * g, Dt_t * list,
 #ifndef WITH_CGRAPH
 	n = agnode(vg, agnameof(p->np));     /* FIX */
 #else
-	n = agnode(cg, agnameof(p->np), 1);  /* FIX */
-	agbindrec(cg, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);  //node custom data
-
+	n = agnode(vg, agnameof(p->np), 1);  /* FIX */
+	agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);  //node custom data
 #endif
 	p->vnode = n;
 	ND_alg(n) = p;
@@ -489,7 +490,7 @@ static graph_t *mkConstraintG(graph_t * g, Dt_t * list,
 	elist_append(e, ND_out(n));
 	elist_append(e, ND_in(vn));
     }
-#endif
+#endif  /* OLD */
 
     return cg;
 }
