@@ -1,4 +1,4 @@
-#! /bin/ksh
+#!/bin/ksh
 #
 # Graphviz regression test driver
 #
@@ -29,7 +29,7 @@ REFDIR=nshare          # Directory for expected test output
 GENERATE=              # If set, generate test data
 VERBOSE=               # If set, give verbose output
 NOOP=                  # If set, just print list of tests
-DOT=../cmd/dot/dot_builtins # build tree version with a builtin set of plugins
+DOT=../cmd/dot/dot # build tree version with a builtin set of plugins
 		       # should be $(top_builddir)/cmd/dot/dot_static
 DIFFIMG=../contrib/diffimg/diffimg # build tree version
 
@@ -387,7 +387,7 @@ then
   exit 1
 fi
 
-if [[ ! -x $DIFFIMG ]]
+if [[ $GENERATE != 1 && ! -x $DIFFIMG ]]
 then
   print -u 2 "$DIFFIMG program is not executable"
   exit 1
@@ -400,4 +400,9 @@ while readTest
 do
   doTest
 done
+if [[ $NOOP != 1 ]]
+then
 print -u 2 "No. tests: $TOT_CNT Layout failures: $CRASH_CNT Changes: $DIFF_CNT"
+else
+print -u 2 "No. tests: $TOT_CNT"
+fi
